@@ -8,6 +8,7 @@ if TYPE_CHECKING:
 
     from narwhals._compliant.dataframe import (
         CompliantDataFrame,
+        CompliantFrame,
         CompliantLazyFrame,
         EagerDataFrame,
     )
@@ -15,6 +16,7 @@ if TYPE_CHECKING:
         CompliantExpr,
         DepthTrackingExpr,
         EagerExpr,
+        ImplExpr,
         LazyExpr,
         NativeExpr,
     )
@@ -23,6 +25,9 @@ if TYPE_CHECKING:
     from narwhals._compliant.window import WindowInputs
     from narwhals.typing import (
         FillNullStrategy,
+        IntoLazyFrame,
+        ModeKeepStrategy,
+        NativeDataFrame,
         NativeFrame,
         NativeSeries,
         RankMethod,
@@ -41,6 +46,7 @@ if TYPE_CHECKING:
         half_life: float | None
         ignore_nulls: bool
         interpolation: RollingInterpolationMethod
+        keep: ModeKeepStrategy
         limit: int | None
         method: RankMethod
         min_samples: int
@@ -61,7 +67,6 @@ __all__ = [
     "CompliantSeriesT",
     "EvalNames",
     "EvalSeries",
-    "IntoCompliantExpr",
     "NarwhalsAggregation",
     "NativeFrameT_co",
     "NativeSeriesT_co",
@@ -71,9 +76,11 @@ CompliantSeriesAny: TypeAlias = "CompliantSeries[Any]"
 CompliantSeriesOrNativeExprAny: TypeAlias = "CompliantSeriesAny | NativeExpr"
 CompliantDataFrameAny: TypeAlias = "CompliantDataFrame[Any, Any, Any, Any]"
 CompliantLazyFrameAny: TypeAlias = "CompliantLazyFrame[Any, Any, Any]"
-CompliantFrameAny: TypeAlias = "CompliantDataFrameAny | CompliantLazyFrameAny"
+CompliantFrameAny: TypeAlias = "CompliantFrame[Any, Any, Any]"
 CompliantNamespaceAny: TypeAlias = "CompliantNamespace[Any, Any]"
 
+ImplExprAny: TypeAlias = "ImplExpr[Any, Any]"
+
 DepthTrackingExprAny: TypeAlias = "DepthTrackingExpr[Any, Any]"
 
 EagerDataFrameAny: TypeAlias = "EagerDataFrame[Any, Any, Any, Any]"
@@ -90,6 +97,8 @@ NativeSeriesT_co = TypeVar("NativeSeriesT_co", bound="NativeSeries", covariant=T
 NativeSeriesT_contra = TypeVar(
     "NativeSeriesT_contra", bound="NativeSeries", contravariant=True
 )
+NativeDataFrameT = TypeVar("NativeDataFrameT", bound="NativeDataFrame")
+NativeLazyFrameT = TypeVar("NativeLazyFrameT", bound="IntoLazyFrame")
 NativeFrameT = TypeVar("NativeFrameT", bound="NativeFrame")
 NativeFrameT_co = TypeVar("NativeFrameT_co", bound="NativeFrame", covariant=True)
 NativeFrameT_contra = TypeVar(
@@ -130,7 +139,7 @@ CompliantNamespaceT_co = TypeVar(
     "CompliantNamespaceT_co", bound=CompliantNamespaceAny, covariant=True
 )
 
-IntoCompliantExpr: TypeAlias = "CompliantExpr[CompliantFrameT, CompliantSeriesOrNativeExprT_co] | CompliantSeriesOrNativeExprT_co"
+ImplExprT_contra = TypeVar("ImplExprT_contra", bound=ImplExprAny, contravariant=True)
 
 DepthTrackingExprT = TypeVar("DepthTrackingExprT", bound=DepthTrackingExprAny)
 DepthTrackingExprT_contra = TypeVar(
@@ -176,6 +185,7 @@ NarwhalsAggregation: TypeAlias = Literal[
     "median",
     "max",
     "min",
+    "mode",
     "std",
     "var",
     "len",
@@ -194,3 +204,8 @@ Be sure to update me if you're working on one of these:
 - https://github.com/narwhals-dev/narwhals/issues/2526
 - https://github.com/narwhals-dev/narwhals/issues/2660
 """
+
+Accessor: TypeAlias = Literal[
+    "arr", "cat", "dt", "list", "meta", "name", "str", "bin", "struct"
+]
+"""`{Expr,Series}` method namespace accessor name."""
